#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stddef.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <regex.h>
#include <errno.h>

#define VERSION 1.2

/* MAILCMD should be a valid shell command for sending mail that
 * can use STDIN for input.				*/
#define MAILCMD "mail -s \"FIREWALL LOG DAEMON\" "
#define MAILTARGET "root"
/* the above two defines end up getting concatenated if email address is
 * not specified on the command line */

/* LOGFILE should be a named pipe (FIFO) that syslog will push
 * kernel logs through.
 * Create the fifo using the command:
 * 	mkfifo /var/log/kernelpipe
 * 
 * Edit /etc/syslog.conf and include a line that will push logs
 * through the pipe. Add the following line:
 * 	kern.info		|/var/log/kernelpipe
 *
 * Restart syslogd and you are ready to go!		*/
#define LOGFILE "/var/log/kernelpipe"

/* BUFFERSIZE is the default number of log entries to queue before
 * sending them by mail. Set this to whatever is reasonable for you.
 * This number can be overridden by a command line switch. */
#define BUFFERSIZE 10


#define PIDFILE "/var/run/firelogd.pid"
#define BUFFERFILE "/var/tmp/firelogd.buffer"
#define TEMPLATE "/etc/firelog.conf"
#define SVCFILE "/etc/iana-port-numbers"
#define ICMPFILE "/etc/iana-icmp-numbers"
#define USE_EXT_SVC 1
#define USE_STD_SVC 2


struct pktlog
{
  char date[16];
  char msg[96];
  char iface[5];
  char srcip[16];
  char srcip_res[255];
  int srcpt;
  char dstip[16];
  int dstpt;
  char dstpt_res[16];
  char proto[16];
  int icmptype;
  int icmpcode;
  char icmp_res[128];
};

int match (const char *string, const regex_t * re, struct pktlog *pkt,
	   const int type);
void quit (int sig);		/* close it down properly */
int readbuf (void);		/* read in the buffer file */
int writebuf (void);		/* write buffer file */
/* resolve icmp types and codes */
int icmp_resolve (int icmptype, int icmpcode, char *icmpst);
int resolve_pkt (struct pktlog *pkt);
/* format for output */
int fmt_prn (FILE * out, const struct pktlog *pkt, int preproc_c, char **preproc_v);
void pkt_str(const char *inst, char *outst, const struct pktlog *pkt);	/* returns string for
						   	string matching var name
							struct pktlog	*/
void usage (char *app);		/* usage and die */
