#include "dmn.h"

void
usage (char *app)
{
  printf ("\n%s - firewall log watch daemon - ", app);
  printf ("version %.1f\nIan Jones - roux@speakeasy.org\n\n", VERSION);
  printf
    ("Usage:\n\t %s [-k] [-d] [-m] [-s] [-b<buffersize>] [-l<logfile>]\n\t\t  [-t<template>] [-e<mail>] [-]\n\n",
     app);
  printf ("Use without any options to print to STDOUT.\n\n");
  exit (1);
}


void
quit (int sig)
{
  extern int dmn;
  if (!dmn)
    exit (sig);
  writebuf ();
  unlink (PIDFILE);
  exit (sig);
}

int
readbuf (void)
{
  FILE *buffer;
  extern int buf_counter;
  extern int buffersize;
  extern struct pktlog *mailbuffer;
  if ((buffer = fopen (BUFFERFILE, "r")) != NULL)
    {
      buf_counter =
	fread (mailbuffer, sizeof (struct pktlog), buffersize, buffer);
      fclose (buffer);
      unlink (BUFFERFILE);
    }
  return buf_counter;
}

int
writebuf (void)
{
  FILE *buffer;
  extern int buf_counter;
  extern struct pktlog *mailbuffer;
  int wrote = 0;
  if ((buffer = fopen (BUFFERFILE, "w")) == NULL)
    {
      return 0;
    }
  else
    {
      if (buf_counter != 0)
	{
	  wrote =
	    fwrite (mailbuffer, sizeof (struct pktlog), buf_counter, buffer);
	}
    }
  fclose (buffer);
  return wrote;
}

int
fmt_prn (FILE * out, const struct pktlog *pkt, int preproc_c,
	 char **preproc_v)
{
  int printed = 0;
  int counter = 0;
  char outst[255] = "";

  if (preproc_c)
    {
      for (counter = 0; counter < preproc_c; counter++)
	{
	  pkt_str (preproc_v[counter], outst, pkt);
	  if (strcmp (outst, "0") == 0)
	    continue;
	  fputs (outst, out);
	  strcpy (outst, "");
	}
      printed++;
    }
  else
    {
      if (strcmp (pkt->proto, "ICMP") == 0)
	{
	  printed =
	    (fprintf
	     (out, "%s  %s  LOG: %s\nTYPE:%d/CODE:%d - %s\n%s -> %s\n%s\n\n",
	      pkt->date, pkt->proto, pkt->msg, pkt->icmptype, pkt->icmpcode,
	      pkt->icmp_res, pkt->srcip, pkt->dstip, pkt->srcip_res));
	}
      else
	{
	  printed =
	    (fprintf
	     (out, "%s  %s  LOG: %s\n%s:%d -> %s:%d\n%s -> %s\n\n", pkt->date,
	      pkt->proto, pkt->msg, pkt->srcip, pkt->srcpt, pkt->dstip,
	      pkt->dstpt, pkt->srcip_res, pkt->dstpt_res));
	}
    }
  return printed;
}

void
pkt_str (const char *inst, char *outst, const struct pktlog *pkt)
{
  if (strcmp (inst, "icmp_res") == 0)
    strcpy (outst, pkt->icmp_res);
  else if (strcmp (inst, "icmpcode") == 0)
    sprintf (outst, "%d", pkt->icmpcode);
  else if (strcmp (inst, "icmptype") == 0)
    sprintf (outst, "%d", pkt->icmptype);
  else if (strcmp (inst, "proto") == 0)
    strcpy (outst, pkt->proto);
  else if (strcmp (inst, "dstpt_res") == 0)
    strcpy (outst, pkt->dstpt_res);
  else if (strcmp (inst, "dstpt") == 0)
    sprintf (outst, "%d", pkt->dstpt);
  else if (strcmp (inst, "dstip") == 0)
    strcpy (outst, pkt->dstip);
  else if (strcmp (inst, "srcpt") == 0)
    sprintf (outst, "%d", pkt->srcpt);
  else if (strcmp (inst, "srcip_res") == 0)
    strcpy (outst, pkt->srcip_res);
  else if (strcmp (inst, "srcip") == 0)
    strcpy (outst, pkt->srcip);
  else if (strcmp (inst, "iface") == 0)
    strcpy (outst, pkt->iface);
  else if (strcmp (inst, "msg") == 0)
    strcpy (outst, pkt->msg);
  else if (strcmp (inst, "date") == 0)
    strcpy (outst, pkt->date);
  else if (strcmp (inst, "nl") == 0)
    strcpy (outst, "\n");
  else if (strcmp (inst, "sp") == 0)
    strcpy (outst, " ");
  else
    strcpy (outst, inst);
}
