#include <stdio.h>
#include <stdlib.h>
#include <regex.h>
#include "match.h"
#include "dmn.h"

/* This takes the input <string>, the precompiled regular	*
 * expression <re>, the address of a pktlog structure and the 	*
 * constant <MAT_CHAIN || MAT_TABLE>.				*
 *								*
 * It returns (1) for success, (0) for failure			*
 */

int
match (const char *string,
       const regex_t * re, struct pktlog *pkt, const int type)
{
  int status;
  int isicmp = 0;
  unsigned len;
  unsigned count;
  unsigned safety = 2048;
  char out[safety];
  char tmp[safety];
  size_t nmatch = 12;
  regmatch_t matches[12];

  /*
     struct servent *new_pt;
     struct hostent *new_hst;
     struct in_addr hst;
   */
  struct protoent *newproto;

  status = regexec (re, string, nmatch, matches, 0);
  if (status != 0)
    return (0);
  if (type == MAT_TABLE)
    {
      for (count = 1; count < nmatch; count++)
	{
	  len = matches[count].rm_eo - matches[count].rm_so;
	  if (len >= safety)
	    return (0);
	  strncpy (out, string + matches[count].rm_so, len);
	  out[len] = 0;
	  switch (count)
	    {
	    case 1:
	      if (strlen (out) >= sizeof (pkt->date))
		return (0);
	      strcpy (pkt->date, out);
	      break;
	    case 2:
	      if (strlen (out) >= sizeof (pkt->msg))
		return (0);
	      strcpy (pkt->msg, out);
	      break;
	    case 3:
	      if (strlen (out) >= sizeof (pkt->iface))
		return (0);
	      strcpy (pkt->iface, out);
	      break;
	    case 4:
	      if (strlen (out) >= sizeof (pkt->srcip))
		return (0);
	      strcpy (pkt->srcip, out);
	      strcpy (pkt->srcip_res, out);
	      break;
	    case 5:
	      if (strlen (out) >= sizeof (pkt->dstip))
		return (0);
	      strcpy (pkt->dstip, out);
	      break;
	    case 6:
	      if (strlen (out) >= sizeof (pkt->proto))
		return (0);
	      strcpy (pkt->proto, out);
	      if (strcmp (pkt->proto, "ICMP") == 0)
		isicmp = 1;
	      break;
	    case 7:
	      break;
	    case 8:
	      if (isicmp)
		pkt->icmptype = atoi (out);
	      break;
	    case 9:
	      if (isicmp)
		pkt->icmpcode = atoi (out);
	      break;
	    case 10:
	      pkt->srcpt = atoi (out);
	      break;
	    case 11:
	      if (strlen (out) >= sizeof (pkt->dstpt_res))
		return (0);
	      pkt->dstpt = atoi (out);
	      strcpy (pkt->dstpt_res, out);
	      break;
	    default:
	      continue;
	    }
	}
    }
  else if (type == MAT_CHAIN)
    {
      for (count = 1; count < nmatch - 2; count++)
	{
	  len = matches[count].rm_eo - matches[count].rm_so;
	  if (len >= safety)
	    return (0);
	  strncpy (out, string + matches[count].rm_so, len);
	  out[len] = 0;
	  switch (count)
	    {
	    case 1:
	      if (strlen (out) >= sizeof (pkt->date))
		return (0);
	      strcpy (pkt->date, out);
	      break;
	    case 2:
	      if (strlen (out) >= sizeof (tmp))
		return (0);
	      strcpy (tmp, out);
	      break;
	    case 3:
	      if (strlen (out) >= (sizeof (pkt->msg) - sizeof (tmp)) - 4)
		return (0);
	      strcpy (pkt->msg, out);
	      strcat (pkt->msg, " in ");
	      strcat (pkt->msg, tmp);
	      break;
	    case 4:
	      if (strlen (out) >= sizeof (pkt->iface))
		return (0);
	      strcpy (pkt->iface, out);
	      break;
	    case 5:
	      isicmp = atoi (out);
	      newproto = getprotobynumber (isicmp);
	      if (strlen (newproto->p_name) >= sizeof (pkt->proto))
		return (0);
	      strcpy (pkt->proto, newproto->p_name);
	      if (isicmp != 1)
		isicmp = 0;	/* if not icmp, reset to 0 */
	      break;
	    case 6:
	      if (strlen (out) >= sizeof (pkt->srcip))
		return (0);
	      strcpy (pkt->srcip, out);
	      strcpy (pkt->srcip_res, out);
	      break;
	    case 7:
	      if (isicmp)
		{
		  pkt->icmptype = atoi (out);
		  pkt->srcpt = 0;
		}
	      else
		pkt->srcpt = atoi (out);
	      break;
	    case 8:
	      if (strlen (out) >= sizeof (pkt->dstip))
		return (0);
	      strcpy (pkt->dstip, out);
	      break;
	    case 9:
	      if (isicmp)
		{
		  pkt->icmpcode = atoi (out);
		  pkt->dstpt = 0;
		}
	      else
		{
		  pkt->dstpt = atoi (out);
		  if (strlen (out) >= sizeof (pkt->dstpt_res))
		    return (0);
		  strcpy (pkt->dstpt_res, out);
		}
	      break;
	    default:
	      continue;
	    }
	}
    }
  else
    return (0);
  if (resolve_pkt (pkt))
    return (1);
  else
    return (0);
}
